from .ahnonlosoio import *
from .answer import *
from .cat import *
from .ciaoruozi import *
from .color import *
from .ping import *
from .ship import *
from .emojify import *
from .dog import *
from .fortune import *
from .pmots import *
from .spell import *
from .smecds import *
from .man import *
from .login import *
from .whoami import *
from .fiorygi import *
from .version import *
from .cv import *
